@@ -25,6 +25,7 @@ import Control.Arrow
 import Control.Applicative ( (<$>) )
 import Control.Monad.State
 import Data.Char
+import Data.List
 import qualified Data.Map as M
 import Data.Maybe
 
@@ -110,20 +111,15 @@ evalElement el
                                             (appendOutput fm)
     | Substitute (e:els)    <- el = ifEmpty (consuming $ substituteWith e)
                                             (getFirst els) id
-    | ShortNames   s fm d   <- el = head <$> gets (names . env) >>= \(Names _ ns fm' d' _) ->
-                                    appendOutput fm' <$> evalNames False s (updateNameParts d fm ns) d'
     | otherwise                   = return []
     where
       substituteWith e = head <$> gets (names . env) >>= \(Names _ ns fm d _) -> do
                            case e of
-                             Names rs [Name NotSet _ [] [] []] fm' d' []
-                                 -> evalElement $ Names rs ns (mergeFM fm' fm) (d' `betterThen` d) []
+                             Names rs [Name NotSet fm'' [] [] []] fm' d' []
+                                 -> let nfm = mergeFM fm'' $ mergeFM fm' fm in
+                                    evalElement $ Names rs ns nfm (d' `betterThen` d) []
                              _   -> evalElement e
 
-      updateNameParts d fm (Name f fm' nf d' np : xs) = Name f (mergeFM fm' fm) nf (d `betterThen` d') np : xs
-      updateNameParts d fm                   (x : xs) = x : updateNameParts d fm xs
-      updateNameParts _ _                         []  = []
-
       tryGroup     l = get >>= \s -> evalElements (rmTermConst l) >>= \r -> put s >> return r
       rmTermConst [] = []
       rmTermConst (e:es)
@@ -153,7 +149,12 @@ evalElement el
                              "title"       -> formatTitle f fm
                              "locator"     -> getLocVar >>= formatRange fm . snd
                              "url"         -> getStringVar "url" >>= \k ->
-                                              if null k then return [] else return [OUrl k fm]
+                                              if null k then return [] else return [OUrl (k,k) fm]
+                             "doi"         -> getStringVar "doi" >>= \d ->
+                                              if "doi:" `isPrefixOf` d
+                                                 then let d' = drop 4 d in
+                                                      return [OUrl ("http://dx.doi.org/" ++ d', d') fm]
+                                                 else return [OStr d  fm]
                              _             -> gets (env >>> options &&& abbrevs) >>= \(opts,as) ->
                                               getVar [] (getFormattedValue opts as f fm s) s >>= \r ->
                                               consumeVariable s >> return r
