@@ -117,16 +117,19 @@ formatNames ea del p s as n
         return res
 
     | EtAl fm t <- n = do
-        et <- gets etal
         o <- gets (options . env)
-        let i = length as - length et
-            t' = if null t then "et-al" else t
-        r <- mapM (et_al o False t' fm del) [i .. length as]
-        let (r',r'') = case r of
-                         (x:xs) -> ( x,xs ++ [])
-                         _      -> ([],      [])
-        updateEtal r''
-        return r'
+        if (getOptionVal "et-al-min" o == [])
+           then return []
+           else do
+             et <- gets etal
+             let i = length as - length et
+                 t' = if null t then "et-al" else t
+             r <- mapM (et_al o False t' fm del) [i .. length as]
+             let (r',r'') = case r of
+                              (x:xs) -> ( x,xs ++ [])
+                              _      -> ([],      [])
+             updateEtal r''
+             return r'
 
     | otherwise = return []
     where
