@@ -29,7 +29,11 @@ appendOutput :: Formatting -> [Output] -> [Output]
 appendOutput fm xs = if xs /= [] then [Output xs fm] else []
 
 outputList :: Formatting -> Delimiter -> [Output] -> [Output]
-outputList fm d = appendOutput fm . addDelim d
+outputList fm d = appendOutput fm . addDelim d . map cleanOutput'
+    where
+      cleanOutput' o
+          | Output xs f <- o = Output (cleanOutput xs) f
+          | otherwise        = rmEmptyOutput o
 
 cleanOutput :: [Output] -> [Output]
 cleanOutput = flatten
@@ -37,12 +41,16 @@ cleanOutput = flatten
       flatten [] = []
       flatten (o:os)
           | ONull       <- o     = flatten os
-          | Output [] _ <- o     = flatten os
-          | OStr []   _ <- o     = flatten os
-          | OUrl []   _ <- o     = flatten os
           | Output xs f <- o
           , f == emptyFormatting = flatten xs ++ flatten os
-          | otherwise            = o : flatten os
+          | otherwise            = rmEmptyOutput o : flatten os
+
+rmEmptyOutput :: Output -> Output
+rmEmptyOutput o
+    | Output [] _ <- o = ONull
+    | OStr []   _ <- o = ONull
+    | OUrl t    _ <- o = if null (fst t) then ONull else o
+    | otherwise        = o
 
 addDelim :: String -> [Output] -> [Output]
 addDelim d = foldr (\x xs -> if length xs < 1 then x : xs else check x xs) []
