@@ -43,15 +43,16 @@ xpReference :: PU Reference
 xpReference
     = xpWrap ( \ ( ref
                 , (ck,ty,ti,i,d)
-                ,((au,ed,tr),(re,it,pu'),(co,ce,dg,om))
+                ,((au,ed,tr),(re,it,pu',dr),(co,ce,dg,om))
                 ,((di',pg,vl,is),(nu,sc,ch))
                 , (di,ac,pu,pp,et)
                 , ((ac',uri),no)
                  ) ->
-               ref { refId            = ck `betterThen` take 10 (concat $ words ti)
+               ref { refId            = ck `betterThen` take 10 (concat . words $ fst ti)
                    , refType          = if ty /= NoType then ty else
                                         if refType ref == Book then Chapter else refType ref
-                   , title            = ti
+                   , title            = fst ti
+                   , titleShort       = snd ti
                    , author           = au
                    , editor           = ed `betterThen` editor           ref
                    , edition          = et `betterThen` edition          ref
@@ -59,6 +60,7 @@ xpReference
                    , recipient        = re `betterThen` recipient        ref
                    , interviewer      = it `betterThen` interviewer      ref
                    , composer         = co `betterThen` composer         ref
+                   , director         = dr `betterThen` director         ref
                    , collectionEditor = ce `betterThen` collectionEditor ref
                    , publisherPlace   = pp `betterThen` publisherPlace   ref
                    , containerAuthor  = containerAuthor  ref
@@ -82,9 +84,9 @@ xpReference
                                            `betterThen` fromAgent om
                    }
              , \r -> (  r
-                     , (refId     r, refType          r, title          r, isbn r, doi r)
+                     , (refId     r, refType          r, (title r, titleShort r), isbn r, doi r)
                      ,((author    r, editor           r, translator     r)
-                      ,(recipient r, interviewer      r, emptyAgents     )
+                      ,(recipient r, interviewer      r, emptyAgents, director  r)
                       ,(composer  r, collectionEditor r, emptyAgents, emptyAgents))
                      ,((issued    r, page  r, volume r, issue r)
                       ,(number    r, section   r, chapterNumber r))
@@ -130,37 +132,39 @@ xpRelatedItem :: PU Reference
 xpRelatedItem
     = xpIElem "relatedItem" . xpAddFixedAttr "type" "host" $
       xpWrap ( \( (ty,ct)
-                ,((ca,ed,tr),(re,it,pu'),(co,ce,dg,om))
+                ,((ca,ed,tr),(re,it,pu',dr),(co,ce,dg,om))
                 ,((di,pg,vl,is),(nu,sc,ch))
                 , (di',ac,pu,pp,et)
                 ) ->
-               emptyReference { refType          = ty
-                              , containerAuthor  = ca
-                              , containerTitle   = ct
-                              , editor           = ed
-                              , edition          = et
-                              , translator       = tr
-                              , recipient        = re
-                              , interviewer      = it
-                              , publisherPlace   = pp
-                              , composer         = co
-                              , collectionEditor = ce
-                              , issued           = di `betterThen` di'
-                              , accessed         = ac
-                              , page             = pg
-                              , volume           = vl
-                              , issue            = is `betterThen` nu
-                              , number           = nu
-                              , section          = sc
-                              , chapterNumber    = ch
-                              , publisher        = fromAgent $ pu `betterThen` pu' `betterThen`
-                                                               dg `betterThen` om
+               emptyReference { refType             = ty
+                              , containerAuthor     = ca
+                              , containerTitle      = fst ct
+                              , containerTitleShort = snd ct
+                              , editor              = ed
+                              , edition             = et
+                              , translator          = tr
+                              , recipient           = re
+                              , interviewer         = it
+                              , publisherPlace      = pp
+                              , composer            = co
+                              , director            = dr
+                              , collectionEditor    = ce
+                              , issued              = di `betterThen` di'
+                              , accessed            = ac
+                              , page                = pg
+                              , volume              = vl
+                              , issue               = is `betterThen` nu
+                              , number              = nu
+                              , section             = sc
+                              , chapterNumber       = ch
+                              , publisher           = fromAgent $ pu `betterThen` pu' `betterThen`
+                                                                  dg `betterThen` om
 
                               }
-             , \r -> ( (refType         r, containerTitle   r)
+             , \r -> ( (refType         r, (containerTitle  r, containerTitleShort r))
                      ,((containerAuthor r, editor           r, translator r)
-                      ,(recipient       r, interviewer      r, emptyAgents )
-                      ,(composer        r, collectionEditor r, emptyAgents , emptyAgents))
+                      ,(recipient       r, interviewer      r, emptyAgents, director  r)
+                      ,(composer        r, collectionEditor r, emptyAgents, emptyAgents))
                      ,((issued  r, page  r, volume r, issue r)
                       ,(number  r, section   r, chapterNumber r))
                      , (issued  r, accessed  r,emptyAgents, publisherPlace r, edition r)
@@ -168,24 +172,33 @@ xpRelatedItem
       xp4Tuple (xpPair  xpRefType xpTitle)
                 xpAgents xpPart xpOrigin
 
--- FIXME: join title and subtitle correctly: usare Title per shortTitle.
-xpTitle :: PU String
+xpTitle :: PU (String,String)
 xpTitle
-    = xpWrap (uncurry (<+>), \s -> (s,[])) $
-      xpIElem "titleInfo" $
-      xpPair (xpIElem "title" xpText0)
-             (xpDefault [] $ xpIElem "subTitle" xpText0)
+    = xpWrap (\((a,b),c) -> createTitle a b c , \s -> (s,[])) $
+      xpPair (xpIElem "titleInfo" $
+              xpPair (xpIElem "title" xpText0)
+                     (xpDefault [] $ xpIElem "subTitle" xpText0))
+             (xpDefault [] $ xpIElem "titleInfo" $
+              xpAddFixedAttr "type" "abbreviated" $ xpElem "title" xpText0)
+    where
+      createTitle [] [] [] = ([],[])
+      createTitle s  [] [] = (s ,[])
+      createTitle s  [] ab = (s ,ab)
+      createTitle s sub [] = (s ++ colon s ++ sub,  s)
+      createTitle s sub ab = (s ++ colon s ++ sub, ab)
+      colon s = if last s == '!' || last s == '?' then " " else ": "
 
 xpAgents :: PU (([Agent],[Agent],[Agent])
-               ,([Agent],[Agent],[Agent])
+               ,([Agent],[Agent],[Agent],[Agent])
                ,([Agent],[Agent],[Agent],[Agent]))
 xpAgents
     = xpTriple (xpTriple (xpAgent "author"         "aut")
                          (xpAgent "editor"         "edt")
                          (xpAgent "translator"     "trl"))
-               (xpTriple (xpAgent "recipient"      "rcp")
+               (xp4Tuple (xpAgent "recipient"      "rcp")
                          (xpAgent "interviewer"    "ivr")
-                         (xpAgent "publisher"      "pbl"))
+                         (xpAgent "publisher"      "pbl")
+                         (xpAgent "director"       "drt"))
                (xp4Tuple (xpAgent "composer"       "cmp")
                          (xpAgent "collector"      "xol")
                          (xpAgent "degree grantor" "dgg")
@@ -206,7 +219,7 @@ instance XmlPickler Agent where
           personal = xpWrap ( uncurry parseName
                             , \(Agent gn _ _ fn _ _ _) -> (gn,fn)) $
                      xpAddFixedAttr "type" "personal" xpNameData
-          others   =xpWrap (\s -> Agent [] [] [] [] [] s False, undefined) $
+          others   = xpWrap (\s -> Agent [] [] [] [] [] s False, undefined) $
                      xpElem "namePart" xpText0
 
 -- | "von Hicks,! Jr., Michael" or "la Martine,! III, Martin B. de" or
@@ -276,6 +289,7 @@ xpDetail
            , xpWrap (Volume,   show) $ xp "volume"
            , xpWrap (Issue,    show) $ xp "issue"
            , xpWrap (Number,   show) $ xp "number"
+           , xpWrap (Number,   show) $ xp "report number"
            , xpWrap (Section,  show) $ xp "section"
            , xpWrap (ChapterNr,show) $ xp "chapter"
            ]
@@ -346,6 +360,7 @@ genreTypeMapping = M.fromList
   , ( "book chapter",               Chapter          )
   , ( "periodical",                 ArticleJournal   )
   , ( "newspaper",                  ArticleNewspaper )
+  , ( "magazine article",           ArticleNewspaper )
   , ( "encyclopedia",               EntryEncyclopedia)
   , ( "conference publication",     PaperConference  )
   , ( "academic journal",           ArticleJournal   )
@@ -353,7 +368,11 @@ genreTypeMapping = M.fromList
   , ( "legal case and case notes",  LegalCase        )
   , ( "legislation",                Legislation      )
   , ( "instruction",                Book             )
-  , ( "motion picutre",             MotionPicture    )
+  , ( "motion picture",             MotionPicture    )
+  , ( "film",                       MotionPicture    )
+  , ( "tvBroadcast",                MotionPicture    )
+  , ( "videoRecording",             MotionPicture    )
+  , ( "videorecording",             MotionPicture    )
   , ( "patent",                     Patent           )
   , ( "Ph.D. thesis",               Thesis           )
   , ( "Masters thesis",             Thesis           )
