@@ -23,7 +23,6 @@ import Text.CSL.Pickle
 #ifdef EMBED_DATA_FILES
 import Data.FileEmbed
 import qualified Data.ByteString as S
-import Data.ByteString.UTF8 ( toString )
 #else
 import Paths_citeproc_hs ( getDataFileName )
 import System.Directory  ( doesFileExist   )
@@ -69,7 +68,7 @@ parseCSL' f = do
   return s { styleLocale = mergeLocales (styleDefaultLocale s) l (styleLocale s)}
 
 #ifdef EMBED_DATA_FILES
-localeFiles :: [(FilePath, L.ByteString)]
+localeFiles :: [(FilePath, S.ByteString)]
 localeFiles = $(embedDir "locales/")
 #endif
 
@@ -86,7 +85,7 @@ readLocaleFile s = do
             | otherwise     -> case lookup ("locales-" ++ take 5 x ++ ".xml") localeFiles of
                                  Just x' -> return x'
                                  _       -> error "could not load the locale file"
-  return $ readXmlString xpLocale f
+  return $ readXmlString xpLocale $ L.fromChunks [f]
 #else
   f <- case s of
          x | length x == 2 -> getDataFileName ("locales/locales-" ++
@@ -114,7 +113,6 @@ instance XmlPickler Element where
           tag (Term         {}) =  5
           tag (Label        {}) =  6
           tag (Names        {}) =  7
-          tag (ShortNames   {}) =  8
           tag (Substitute   {}) =  9
           tag (Group        {}) = 10
           tag (Number       {}) = 11
@@ -126,7 +124,6 @@ instance XmlPickler Element where
                , xpTerm
                , xpLabel
                , xpNames
-               , xpShortNames
                , xpSubStitute
                , xpGroup
                , xpNumber
@@ -472,13 +469,6 @@ xpNames
           check [] = xpLift [Name NotSet emptyFormatting [] [] []]
           check  l = if any isName l then xpLift l else xpZero
 
-xpShortNames :: PU Element
-xpShortNames
-    = xpWrap ( \((a,fm),d)          -> ShortNames (words a) fm d
-             , \(ShortNames a fm d) -> ((unwords a,fm),d)) $
-      xpElem "names" $ xpPair content xpDelimiter
-    where content = xpPair (xpAttrText "variable") xpickle
-
 xpLabel :: PU Element
 xpLabel
     = xpWrap ( uncurry4 Label
