@@ -20,7 +20,7 @@ import Data.Generics ( Typeable, Data, everywhere
                      , everywhere', everything, mkT, mkQ)
 import qualified Data.Map as M
 import Text.JSON
-import Text.Pandoc.Definition ( Inline )
+import Text.Pandoc.Definition ( Inline, Target )
 
 -- | The representation of a parsed CSL style.
 data Style
@@ -108,7 +108,6 @@ data Element
     | Term         String     Form        Formatting Bool
     | Label        String     Form        Formatting Plural
     | Number       String     NumericForm Formatting
-    | ShortNames  [String]                Formatting Delimiter
     | Names       [String]   [Name]       Formatting Delimiter [Element]
     | Substitute  [Element]
     | Group        Formatting Delimiter  [Element]
@@ -286,10 +285,10 @@ mergeFM (Formatting aa ab ac ad ae af ag ah ai aj ak al am an)
                               (bh `betterThen` ah)
                               (bi `betterThen` ai)
                               (bj `betterThen` aj)
-                              (if bk /= ak then bk else ak)
-                              (if bl /= al then bl else al)
-                              (if bm /= am then bm else am)
-                              (if bn /= an then bn else an)
+                              (bk || ak)
+                              (bl || al)
+                              (bm || am)
+                              (bn || an)
 
 data CSInfo
     = CSInfo
@@ -309,8 +308,8 @@ data FormattedOutput
     = FO          Formatting [FormattedOutput]          -- ^ List of 'FormatOutput' items
     | FN   String Formatting                            -- ^ Formatted number
     | FS   String Formatting                            -- ^ Formatted string
-    | FUrl String Formatting                            -- ^ Formatted uniform resource locator (URL)
     | FDel String                                       -- ^ Delimeter string
+    | FUrl Target Formatting                            -- ^ Formatted URL
     | FPan [Inline]                                     -- ^ Pandoc inline elements
     | FNull                                             -- ^ Null formatting item
       deriving ( Eq, Show )
@@ -325,13 +324,14 @@ data Output
     | OStr     String             Formatting            -- ^ A simple 'String'
     | ONum     Int                Formatting            -- ^ A number (used to count contributors)
     | OCitNum  Int                Formatting            -- ^ The citation number
+    | ODate   [Output]                                  -- ^ A (possibly) ranged date
     | OYear    String    String   Formatting            -- ^ The year and the citeId
     | OYearSuf String    String   [Output]   Formatting -- ^ The year suffix, the citeId and a holder for collision data
     | OName    String   [Output] [[Output]]  Formatting -- ^ A (family) name with the list of given names.
     | OContrib String    String   [Output] [Output] [[Output]] -- ^ The citation key, the role (author, editor, etc.), the contributor(s),
                                                         -- the output needed for year suf. disambiguation, and everything used for
                                                         -- name disambiguation.
-    | OUrl    String              Formatting            -- ^ A uniform resource locator (URL)
+    | OUrl    Target              Formatting            -- ^ An URL
     | OLoc    [Output]            Formatting            -- ^ The citation's locator
     | Output  [Output]            Formatting            -- ^ Some nested 'Output'
       deriving ( Eq, Ord, Show, Typeable, Data )
@@ -444,6 +444,7 @@ formatOutput o
     | ODel     s         <- o = FDel s
     | OStr     []      _ <- o = FNull
     | OStr     s       f <- o = FS s         f
+    | ODate    os        <- o = FO emptyFormatting (format os)
     | OYear    s _     f <- o = FS s         f
     | OYearSuf s _ _   f <- o = FS s         f
     | ONum     i       f <- o = FS (show  i) f
@@ -501,13 +502,16 @@ addGivenNames
 -- | Add the year suffix to the year. Needed for disambiguation.
 addYearSuffix :: Output -> Output
 addYearSuffix o
-    | OYear y k       f <- o = Output [OYear y k emptyFormatting,OYearSuf [] k [] emptyFormatting] f
-    | Output (x:xs)   f <- o = if or $ map hasYearSuf (x : xs)
-                               then Output (x : xs) f
-                               else if hasYear x
-                                    then Output (addYearSuffix x : xs) f
-                                    else Output (x : [addYearSuffix $ Output xs emptyFormatting]) f
-    | otherwise              = o
+    | OYear y k     f <- o = Output [OYear y k emptyFormatting,OYearSuf [] k [] emptyFormatting] f
+    | ODate  (x:xs)   <- o = if or $ map hasYear xs
+                             then Output (x : [addYearSuffix $ ODate xs]) emptyFormatting
+                             else addYearSuffix (Output (x:xs) emptyFormatting)
+    | Output (x:xs) f <- o = if or $ map hasYearSuf (x : xs)
+                             then Output (x : xs) f
+                             else if hasYear x
+                                  then Output (addYearSuffix x : xs) f
+                                  else Output (x : [addYearSuffix $ Output xs emptyFormatting]) f
+    | otherwise            = o
 
 hasYear :: Output -> Bool
 hasYear = not . null . query getYear
